#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <shlobj.h>
#include "AkelDLL.h"
#include "Resources\Resource.h"


//Defines
#define STRID_SAVEINTERVAL 1
#define STRID_MIN          2
#define STRID_SAVEFOCUS    3
#define STRID_SAVESIMPLE   4
#define STRID_SAVENEAR     5
#define STRID_SAVEDIR      6
#define STRID_OK           7
#define STRID_CANCEL       8

#define ST_SIMPLE  0
#define ST_NEAR    1
#define ST_DIR     2

#ifndef BIF_NEWDIALOGSTYLE
  #define BIF_NEWDIALOGSTYLE 0x0040
#endif


//Functions prototypes
BOOL CALLBACK SetupDlgProcA(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK SetupDlgProcW(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
int CALLBACK BrowseCallbackProcA(HWND hWnd, UINT uMsg, LPARAM lParam, LPARAM lpData);
int CALLBACK BrowseCallbackProcW(HWND hWnd, UINT uMsg, LPARAM lParam, LPARAM lpData);
LRESULT CALLBACK NewMainProcA(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK NewMainProcW(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
VOID CALLBACK TimerProcA(HWND hwnd, UINT uMsg, UINT_PTR idEvent, DWORD dwTime);
VOID CALLBACK TimerProcW(HWND hwnd, UINT uMsg, UINT_PTR idEvent, DWORD dwTime);
void AutoSaveA();
void AutoSaveW();
int SaveFileA(HWND hWndEdit);
int SaveFileW(HWND hWndEdit);
BOOL FileExistsA(char *pFile);
BOOL FileExistsW(wchar_t *wpFile);
char* GetFileNameA(char *pFile);
wchar_t* GetFileNameW(wchar_t *wpFile);
void ReadOptionsA();
void ReadOptionsW();
void SaveOptionsA();
void SaveOptionsW();
char* GetLangStringA(LANGID wLangID, int nStringID);
wchar_t* GetLangStringW(LANGID wLangID, int nStringID);
void InitCommon(PLUGINDATA *pd);
void InitMain();
void UninitMain();
BOOL (WINAPI *SHGetPathFromIDListAPrt)(LPCITEMIDLIST, char *);
BOOL (WINAPI *SHGetPathFromIDListWPrt)(LPCITEMIDLIST, wchar_t *);
LPITEMIDLIST (WINAPI *SHBrowseForFolderAPrt)(LPBROWSEINFOA);
LPITEMIDLIST (WINAPI *SHBrowseForFolderWPrt)(LPBROWSEINFOW);

//Global variables
char szPluginName[MAX_PATH];
wchar_t wszPluginName[MAX_PATH];
HINSTANCE hInstanceDLL;
HWND hMainWnd;
HWND hMdiClient;
HICON hMainIcon;
BOOL bOldWindows;
BOOL bMDI;
LANGID wLangSystem;
BOOL bInitCommon=FALSE;
BOOL bInitMain=FALSE;
char szSaveDir[MAX_PATH]="";
wchar_t wszSaveDir[MAX_PATH]=L"";
DWORD dwSaveTimer=0;
DWORD dwSaveInterval=5;
BOOL bSaveIntervalEnable=TRUE;
BOOL bSaveFocusEnable=FALSE;
int nSaveType=ST_SIMPLE;
int nAppActive=TRUE;
WNDPROCDATA *NewMainProcData=NULL;


//Identification
extern "C" void __declspec(dllexport) DllAkelPadID(PLUGINVERSION *pv)
{
  pv->dwAkelDllVersion=AKELDLL;
  pv->dwExeMinVersion3x=MAKE_IDENTIFIER(3, 7, 4, 0);
  pv->dwExeMinVersion4x=MAKE_IDENTIFIER(4, 4, 0, 0);
  pv->pPluginName="AutoSave";
}

//Plugin extern function
extern "C" void __declspec(dllexport) Settings(PLUGINDATA *pd)
{
  if (pd->lpbAutoLoad)
  {
    *pd->lpbAutoLoad=FALSE;
    return;
  }

  if (!bInitCommon) InitCommon(pd);

  if (bOldWindows)
    DialogBoxA(hInstanceDLL, MAKEINTRESOURCEA(IDD_SETUP), hMainWnd, (DLGPROC)SetupDlgProcA);
  else
    DialogBoxW(hInstanceDLL, MAKEINTRESOURCEW(IDD_SETUP), hMainWnd, (DLGPROC)SetupDlgProcW);

  //If plugin already loaded, stay in memory, but show as non-active
  if (pd->bInMemory) pd->nUnload=UD_NONUNLOAD_NONACTIVE;
}

extern "C" void __declspec(dllexport) Main(PLUGINDATA *pd)
{
  if (pd->lpbAutoLoad)
  {
    *pd->lpbAutoLoad=TRUE;
    return;
  }

  if (!bInitCommon) InitCommon(pd);

  if (bInitMain)
  {
    UninitMain();
  }
  else
  {
    InitMain();

    //Stay in memory, and show as active
    pd->nUnload=UD_NONUNLOAD_ACTIVE;
  }
}

BOOL CALLBACK SetupDlgProcA(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  static HWND hWndSaveIntervalCheck;
  static HWND hWndSaveInterval;
  static HWND hWndSaveFocusCheck;
  static HWND hWndSaveSimpleCheck;
  static HWND hWndSaveNearCheck;
  static HWND hWndSaveDirCheck;
  static HWND hWndSaveDir;
  static HWND hWndSaveDirBrowse;

  if (uMsg == WM_INITDIALOG)
  {
    SendMessage(hDlg, WM_SETICON, (WPARAM)ICON_BIG, (LPARAM)hMainIcon);
    hWndSaveIntervalCheck=GetDlgItem(hDlg, IDC_SAVEINTERVAL_CHECK);
    hWndSaveInterval=GetDlgItem(hDlg, IDC_SAVEINTERVAL);
    hWndSaveFocusCheck=GetDlgItem(hDlg, IDC_SAVEFOCUS_CHECK);
    hWndSaveSimpleCheck=GetDlgItem(hDlg, IDC_SAVESIMPLE_CHECK);
    hWndSaveNearCheck=GetDlgItem(hDlg, IDC_SAVENEAR_CHECK);
    hWndSaveDirCheck=GetDlgItem(hDlg, IDC_SAVEDIR_CHECK);
    hWndSaveDir=GetDlgItem(hDlg, IDC_SAVEDIR);
    hWndSaveDirBrowse=GetDlgItem(hDlg, IDC_SAVEDIR_BROWSE);

    SetDlgItemTextA(hDlg, IDC_SAVEINTERVAL_CHECK, GetLangStringA(wLangSystem, STRID_SAVEINTERVAL));
    SetDlgItemTextA(hDlg, IDC_MIN_LABEL, GetLangStringA(wLangSystem, STRID_MIN));
    SetDlgItemTextA(hDlg, IDC_SAVEFOCUS_CHECK, GetLangStringA(wLangSystem, STRID_SAVEFOCUS));
    SetDlgItemTextA(hDlg, IDC_SAVESIMPLE_CHECK, GetLangStringA(wLangSystem, STRID_SAVESIMPLE));
    SetDlgItemTextA(hDlg, IDC_SAVENEAR_CHECK, GetLangStringA(wLangSystem, STRID_SAVENEAR));
    SetDlgItemTextA(hDlg, IDC_SAVEDIR_CHECK, GetLangStringA(wLangSystem, STRID_SAVEDIR));
    SetDlgItemTextA(hDlg, IDOK, GetLangStringA(wLangSystem, STRID_OK));
    SetDlgItemTextA(hDlg, IDCANCEL, GetLangStringA(wLangSystem, STRID_CANCEL));

    if (bSaveIntervalEnable) SendMessage(hWndSaveIntervalCheck, BM_SETCHECK, BST_CHECKED, 0);
    SendMessage(hWndSaveInterval, EM_LIMITTEXT, 4, 0);
    SetDlgItemInt(hDlg, IDC_SAVEINTERVAL, dwSaveInterval, FALSE);
    if (bSaveFocusEnable) SendMessage(hWndSaveFocusCheck, BM_SETCHECK, BST_CHECKED, 0);

    if (nSaveType == ST_SIMPLE)
      SendMessage(hWndSaveSimpleCheck, BM_SETCHECK, BST_CHECKED, 0);
    else if (nSaveType == ST_NEAR)
      SendMessage(hWndSaveNearCheck, BM_SETCHECK, BST_CHECKED, 0);
    else if (nSaveType == ST_DIR)
      SendMessage(hWndSaveDirCheck, BM_SETCHECK, BST_CHECKED, 0);
    SendMessage(hWndSaveDir, EM_LIMITTEXT, MAX_PATH, 0);
    SetWindowTextA(hWndSaveDir, szSaveDir);

    PostMessage(hDlg, WM_COMMAND, IDC_SAVEINTERVAL_CHECK, 0);
  }
  else if (uMsg == WM_COMMAND)
  {
    if (LOWORD(wParam) == IDC_SAVEINTERVAL_CHECK ||
        LOWORD(wParam) == IDC_SAVEFOCUS_CHECK ||
        LOWORD(wParam) == IDC_SAVESIMPLE_CHECK ||
        LOWORD(wParam) == IDC_SAVENEAR_CHECK ||
        LOWORD(wParam) == IDC_SAVEDIR_CHECK)
    {
      if (!SendMessage(hWndSaveIntervalCheck, BM_GETCHECK, 0, 0) &&
          !SendMessage(hWndSaveFocusCheck, BM_GETCHECK, 0, 0))
      {
        EnableWindow(hWndSaveSimpleCheck, FALSE);
        EnableWindow(hWndSaveNearCheck, FALSE);
        EnableWindow(hWndSaveDirCheck, FALSE);
        EnableWindow(hWndSaveDir, FALSE);
        EnableWindow(hWndSaveDirBrowse, FALSE);
      }
      else
      {
        EnableWindow(hWndSaveSimpleCheck, TRUE);
        EnableWindow(hWndSaveNearCheck, TRUE);
        EnableWindow(hWndSaveDirCheck, TRUE);

        if (SendMessage(hWndSaveDirCheck, BM_GETCHECK, 0, 0) == BST_CHECKED)
        {
          EnableWindow(hWndSaveDir, TRUE);
          EnableWindow(hWndSaveDirBrowse, TRUE);
        }
        else
        {
          EnableWindow(hWndSaveDir, FALSE);
          EnableWindow(hWndSaveDirBrowse, FALSE);
        }
      }
    }
    else if (LOWORD(wParam) == IDC_SAVEDIR_BROWSE)
    {
      BROWSEINFOA bi={0};
      LPITEMIDLIST pIdList;
      LPMALLOC pMalloc;
      char szDir[MAX_PATH];

      GetDlgItemTextA(hDlg, IDC_SAVEDIR, szDir, MAX_PATH);
      bi.hwndOwner=hDlg;
      bi.pszDisplayName=szDir;
      bi.ulFlags=BIF_RETURNONLYFSDIRS|BIF_NEWDIALOGSTYLE;
      bi.lpfn=BrowseCallbackProcA;
      bi.lParam=(LPARAM)szDir;

      if (pIdList=(*SHBrowseForFolderAPrt)(&bi))
      {
        (*SHGetPathFromIDListAPrt)(pIdList, szDir);

        if (SHGetMalloc(&pMalloc))
        {
          pMalloc->Free(pIdList);
          pMalloc->Release();
        }
        SetDlgItemTextA(hDlg, IDC_SAVEDIR, szDir);
      }
      return TRUE;
    }
    else if (LOWORD(wParam) == IDOK)
    {
      bSaveIntervalEnable=SendMessage(hWndSaveIntervalCheck, BM_GETCHECK, 0, 0);
      dwSaveInterval=GetDlgItemInt(hDlg, IDC_SAVEINTERVAL, NULL, FALSE);
      bSaveFocusEnable=SendMessage(hWndSaveFocusCheck, BM_GETCHECK, 0, 0);

      if (SendMessage(hWndSaveSimpleCheck, BM_GETCHECK, 0, 0) == BST_CHECKED)
        nSaveType=ST_SIMPLE;
      else if (SendMessage(hWndSaveNearCheck, BM_GETCHECK, 0, 0) == BST_CHECKED)
        nSaveType=ST_NEAR;
      else if (SendMessage(hWndSaveDirCheck, BM_GETCHECK, 0, 0) == BST_CHECKED)
        nSaveType=ST_DIR;
      GetWindowTextA(hWndSaveDir, szSaveDir, MAX_PATH);

      SaveOptionsA();
      EndDialog(hDlg, 0);

      if (bInitMain)
      {
        UninitMain();
        InitMain();
      }
      return TRUE;
    }
    else if (LOWORD(wParam) == IDCANCEL)
    {
      EndDialog(hDlg, 0);
      return TRUE;
    }
  }
  else if (uMsg == WM_CLOSE)
  {
    PostMessage(hDlg, WM_COMMAND, IDCANCEL, 0);
    return TRUE;
  }
  return FALSE;
}

BOOL CALLBACK SetupDlgProcW(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  static HWND hWndSaveIntervalCheck;
  static HWND hWndSaveInterval;
  static HWND hWndSaveFocusCheck;
  static HWND hWndSaveSimpleCheck;
  static HWND hWndSaveNearCheck;
  static HWND hWndSaveDirCheck;
  static HWND hWndSaveDir;
  static HWND hWndSaveDirBrowse;

  if (uMsg == WM_INITDIALOG)
  {
    SendMessage(hDlg, WM_SETICON, (WPARAM)ICON_BIG, (LPARAM)hMainIcon);
    hWndSaveIntervalCheck=GetDlgItem(hDlg, IDC_SAVEINTERVAL_CHECK);
    hWndSaveInterval=GetDlgItem(hDlg, IDC_SAVEINTERVAL);
    hWndSaveFocusCheck=GetDlgItem(hDlg, IDC_SAVEFOCUS_CHECK);
    hWndSaveSimpleCheck=GetDlgItem(hDlg, IDC_SAVESIMPLE_CHECK);
    hWndSaveNearCheck=GetDlgItem(hDlg, IDC_SAVENEAR_CHECK);
    hWndSaveDirCheck=GetDlgItem(hDlg, IDC_SAVEDIR_CHECK);
    hWndSaveDir=GetDlgItem(hDlg, IDC_SAVEDIR);
    hWndSaveDirBrowse=GetDlgItem(hDlg, IDC_SAVEDIR_BROWSE);

    SetDlgItemTextW(hDlg, IDC_SAVEINTERVAL_CHECK, GetLangStringW(wLangSystem, STRID_SAVEINTERVAL));
    SetDlgItemTextW(hDlg, IDC_MIN_LABEL, GetLangStringW(wLangSystem, STRID_MIN));
    SetDlgItemTextW(hDlg, IDC_SAVEFOCUS_CHECK, GetLangStringW(wLangSystem, STRID_SAVEFOCUS));
    SetDlgItemTextW(hDlg, IDC_SAVESIMPLE_CHECK, GetLangStringW(wLangSystem, STRID_SAVESIMPLE));
    SetDlgItemTextW(hDlg, IDC_SAVENEAR_CHECK, GetLangStringW(wLangSystem, STRID_SAVENEAR));
    SetDlgItemTextW(hDlg, IDC_SAVEDIR_CHECK, GetLangStringW(wLangSystem, STRID_SAVEDIR));
    SetDlgItemTextW(hDlg, IDOK, GetLangStringW(wLangSystem, STRID_OK));
    SetDlgItemTextW(hDlg, IDCANCEL, GetLangStringW(wLangSystem, STRID_CANCEL));

    if (bSaveIntervalEnable) SendMessage(hWndSaveIntervalCheck, BM_SETCHECK, BST_CHECKED, 0);
    SendMessage(hWndSaveInterval, EM_LIMITTEXT, 4, 0);
    SetDlgItemInt(hDlg, IDC_SAVEINTERVAL, dwSaveInterval, FALSE);
    if (bSaveFocusEnable) SendMessage(hWndSaveFocusCheck, BM_SETCHECK, BST_CHECKED, 0);

    if (nSaveType == ST_SIMPLE)
      SendMessage(hWndSaveSimpleCheck, BM_SETCHECK, BST_CHECKED, 0);
    else if (nSaveType == ST_NEAR)
      SendMessage(hWndSaveNearCheck, BM_SETCHECK, BST_CHECKED, 0);
    else if (nSaveType == ST_DIR)
      SendMessage(hWndSaveDirCheck, BM_SETCHECK, BST_CHECKED, 0);
    SendMessage(hWndSaveDir, EM_LIMITTEXT, MAX_PATH, 0);
    SetWindowTextW(hWndSaveDir, wszSaveDir);

    PostMessage(hDlg, WM_COMMAND, IDC_SAVEINTERVAL_CHECK, 0);
  }
  else if (uMsg == WM_COMMAND)
  {
    if (LOWORD(wParam) == IDC_SAVEINTERVAL_CHECK ||
        LOWORD(wParam) == IDC_SAVEFOCUS_CHECK ||
        LOWORD(wParam) == IDC_SAVESIMPLE_CHECK ||
        LOWORD(wParam) == IDC_SAVENEAR_CHECK ||
        LOWORD(wParam) == IDC_SAVEDIR_CHECK)
    {
      if (!SendMessage(hWndSaveIntervalCheck, BM_GETCHECK, 0, 0) &&
          !SendMessage(hWndSaveFocusCheck, BM_GETCHECK, 0, 0))
      {
        EnableWindow(hWndSaveSimpleCheck, FALSE);
        EnableWindow(hWndSaveNearCheck, FALSE);
        EnableWindow(hWndSaveDirCheck, FALSE);
        EnableWindow(hWndSaveDir, FALSE);
        EnableWindow(hWndSaveDirBrowse, FALSE);
      }
      else
      {
        EnableWindow(hWndSaveSimpleCheck, TRUE);
        EnableWindow(hWndSaveNearCheck, TRUE);
        EnableWindow(hWndSaveDirCheck, TRUE);

        if (SendMessage(hWndSaveDirCheck, BM_GETCHECK, 0, 0) == BST_CHECKED)
        {
          EnableWindow(hWndSaveDir, TRUE);
          EnableWindow(hWndSaveDirBrowse, TRUE);
        }
        else
        {
          EnableWindow(hWndSaveDir, FALSE);
          EnableWindow(hWndSaveDirBrowse, FALSE);
        }
      }
    }
    else if (LOWORD(wParam) == IDC_SAVEDIR_BROWSE)
    {
      BROWSEINFOW biW={0};
      LPITEMIDLIST pIdList;
      LPMALLOC pMalloc;
      wchar_t wszDir[MAX_PATH];

      GetDlgItemTextW(hDlg, IDC_SAVEDIR, wszDir, MAX_PATH);
      biW.hwndOwner=hDlg;
      biW.pszDisplayName=wszDir;
      biW.ulFlags=BIF_RETURNONLYFSDIRS|BIF_NEWDIALOGSTYLE;
      biW.lpfn=BrowseCallbackProcW;
      biW.lParam=(LPARAM)wszDir;

      if (pIdList=(*SHBrowseForFolderWPrt)(&biW))
      {
        (*SHGetPathFromIDListWPrt)(pIdList, wszDir);

        if (SHGetMalloc(&pMalloc))
        {
          pMalloc->Free(pIdList);
          pMalloc->Release();
        }
        SetDlgItemTextW(hDlg, IDC_SAVEDIR, wszDir);
      }
      return TRUE;
    }
    else if (LOWORD(wParam) == IDOK)
    {
      bSaveIntervalEnable=SendMessage(hWndSaveIntervalCheck, BM_GETCHECK, 0, 0);
      dwSaveInterval=GetDlgItemInt(hDlg, IDC_SAVEINTERVAL, NULL, FALSE);
      bSaveFocusEnable=SendMessage(hWndSaveFocusCheck, BM_GETCHECK, 0, 0);

      if (SendMessage(hWndSaveSimpleCheck, BM_GETCHECK, 0, 0) == BST_CHECKED)
        nSaveType=ST_SIMPLE;
      else if (SendMessage(hWndSaveNearCheck, BM_GETCHECK, 0, 0) == BST_CHECKED)
        nSaveType=ST_NEAR;
      else if (SendMessage(hWndSaveDirCheck, BM_GETCHECK, 0, 0) == BST_CHECKED)
        nSaveType=ST_DIR;
      GetWindowTextW(hWndSaveDir, wszSaveDir, MAX_PATH);

      SaveOptionsW();
      EndDialog(hDlg, 0);

      if (bInitMain)
      {
        UninitMain();
        InitMain();
      }
      return TRUE;
    }
    else if (LOWORD(wParam) == IDCANCEL)
    {
      EndDialog(hDlg, 0);
      return TRUE;
    }
  }
  else if (uMsg == WM_CLOSE)
  {
    PostMessage(hDlg, WM_COMMAND, IDCANCEL, 0);
    return TRUE;
  }
  return FALSE;
}

int CALLBACK BrowseCallbackProcA(HWND hWnd, UINT uMsg, LPARAM lParam, LPARAM lpData)
{
  if (uMsg == BFFM_INITIALIZED)
  {
    SendMessage(hWnd, BFFM_SETSELECTIONA, TRUE, lpData);
  }
  return 0;
}

int CALLBACK BrowseCallbackProcW(HWND hWnd, UINT uMsg, LPARAM lParam, LPARAM lpData)
{
  if (uMsg == BFFM_INITIALIZED)
  {
    SendMessage(hWnd, BFFM_SETSELECTIONW, TRUE, lpData);
  }
  return 0;
}

LRESULT CALLBACK NewMainProcA(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  if (uMsg == WM_ACTIVATEAPP)
  {
    if (bSaveFocusEnable)
    {
      if (nAppActive != -1)
      {
        if (wParam == FALSE)
        {
          if (nAppActive)
          {
            nAppActive=FALSE;
            AutoSaveA();
          }
        }
        else nAppActive=TRUE;
      }
    }
  }
  else if (uMsg == AKDN_EDIT_ONFINISH)
  {
    EDITINFO ei;
    char szFile[MAX_PATH];
    char *pFileName;

    if (SendMessage(hMainWnd, AKD_GETEDITINFO, wParam, (LPARAM)&ei))
    {
      if (!ei.bModified)
      {
        if (nSaveType == ST_NEAR)
        {
          if (*(char *)ei.pFile)
          {
            wsprintfA(szFile, "%s.tmp", (char *)ei.pFile);
            if (FileExistsA(szFile))
              DeleteFileA(szFile);
          }
        }
        else if (nSaveType == ST_DIR)
        {
          if (*(char *)ei.pFile)
          {
            if (*szSaveDir)
            {
              pFileName=GetFileNameA((char *)ei.pFile);
              wsprintfA(szFile, "%s\\%s", szSaveDir, pFileName);

              if (lstrcmpiA((char *)ei.pFile, szFile))
              {
                if (FileExistsA(szFile))
                  DeleteFileA(szFile);
              }
            }
          }
        }
      }
    }
  }
  else if (uMsg == AKDN_MAIN_ONFINISH)
  {
    if (bSaveFocusEnable)
    {
      nAppActive=-1;
    }
  }

  //Call next procedure
  return NewMainProcData->NextProc(hWnd, uMsg, wParam, lParam);
}

LRESULT CALLBACK NewMainProcW(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  if (uMsg == WM_ACTIVATEAPP)
  {
    if (bSaveFocusEnable)
    {
      if (nAppActive != -1)
      {
        if (wParam == FALSE)
        {
          if (nAppActive)
          {
            nAppActive=FALSE;
            AutoSaveW();
          }
        }
        else nAppActive=TRUE;
      }
    }
  }
  else if (uMsg == AKDN_EDIT_ONFINISH)
  {
    EDITINFO ei;
    wchar_t wszFile[MAX_PATH];
    wchar_t *wpFileName;

    if (SendMessage(hMainWnd, AKD_GETEDITINFO, wParam, (LPARAM)&ei))
    {
      if (!ei.bModified)
      {
        if (nSaveType == ST_NEAR)
        {
          if (*(wchar_t *)ei.pFile)
          {
            wsprintfW(wszFile, L"%s.tmp", (wchar_t *)ei.pFile);
            if (FileExistsW(wszFile))
              DeleteFileW(wszFile);
          }
        }
        else if (nSaveType == ST_DIR)
        {
          if (*(wchar_t *)ei.pFile)
          {
            if (*wszSaveDir)
            {
              wpFileName=GetFileNameW((wchar_t *)ei.pFile);
              wsprintfW(wszFile, L"%s\\%s", wszSaveDir, wpFileName);

              if (lstrcmpiW((wchar_t *)ei.pFile, wszFile))
              {
                if (FileExistsW(wszFile))
                  DeleteFileW(wszFile);
              }
            }
          }
        }
      }
    }
  }
  else if (uMsg == AKDN_MAIN_ONFINISH)
  {
    if (bSaveFocusEnable)
    {
      nAppActive=-1;
    }
  }

  //Call next procedure
  return NewMainProcData->NextProc(hWnd, uMsg, wParam, lParam);
}

VOID CALLBACK TimerProcA(HWND hwnd, UINT uMsg, UINT_PTR idEvent, DWORD dwTime)
{
  AutoSaveA();
}

VOID CALLBACK TimerProcW(HWND hwnd, UINT uMsg, UINT_PTR idEvent, DWORD dwTime)
{
  AutoSaveW();
}

void AutoSaveA()
{
  if (!bMDI)
  {
    SaveFileA(NULL);
  }
  else
  {
    HWND hWndChild;

    hWndChild=GetWindow(hMdiClient, GW_CHILD);

    while (hWndChild)
    {
      if (SaveFileA(GetDlgItem(hWndChild, ID_EDIT))) break;

      hWndChild=GetNextWindow(hWndChild, GW_HWNDNEXT);
    }
  }
}

void AutoSaveW()
{
  if (!bMDI)
  {
    SaveFileW(NULL);
  }
  else
  {
    HWND hWndChild;

    hWndChild=GetWindow(hMdiClient, GW_CHILD);

    while (hWndChild)
    {
      if (SaveFileW(GetDlgItem(hWndChild, ID_EDIT))) break;

      hWndChild=GetNextWindow(hWndChild, GW_HWNDNEXT);
    }
  }
}

int SaveFileA(HWND hWndEdit)
{
  SAVEDOCUMENTA sd;
  EDITINFO ei;

  if (SendMessage(hMainWnd, AKD_GETEDITINFO, (WPARAM)hWndEdit, (LPARAM)&ei))
  {
    if (ei.bModified)
    {
      if (nSaveType == ST_SIMPLE)
      {
        if (*(char *)ei.pFile)
        {
          lstrcpynA(sd.szFile, (char *)ei.pFile, MAX_PATH);
          sd.nCodePage=ei.nCodePage;
          sd.bBOM=ei.bBOM;
          sd.dwFlags=SD_UPDATE;
          return SendMessage(hMainWnd, AKD_SAVEDOCUMENT, (WPARAM)ei.hWndEdit, (LPARAM)&sd);
        }
      }
      else if (nSaveType == ST_NEAR)
      {
        if (*(char *)ei.pFile)
        {
          wsprintfA(sd.szFile, "%s.tmp", (char *)ei.pFile);
          sd.nCodePage=ei.nCodePage;
          sd.bBOM=ei.bBOM;
          sd.dwFlags=0;
          return SendMessage(hMainWnd, AKD_SAVEDOCUMENT, (WPARAM)ei.hWndEdit, (LPARAM)&sd);
        }
      }
      else if (nSaveType == ST_DIR)
      {
        if (*(char *)ei.pFile)
        {
          char *pFileName;

          if (*szSaveDir)
          {
            CreateDirectoryA(szSaveDir, NULL);
            pFileName=GetFileNameA((char *)ei.pFile);
            wsprintfA(sd.szFile, "%s\\%s", szSaveDir, pFileName);
            sd.nCodePage=ei.nCodePage;
            sd.bBOM=ei.bBOM;
            sd.dwFlags=0;
            return SendMessage(hMainWnd, AKD_SAVEDOCUMENT, (WPARAM)ei.hWndEdit, (LPARAM)&sd);
          }
        }
      }
    }
  }
  return 0;
}

int SaveFileW(HWND hWndEdit)
{
  SAVEDOCUMENTW sd;
  EDITINFO ei;

  if (SendMessage(hMainWnd, AKD_GETEDITINFO, (WPARAM)hWndEdit, (LPARAM)&ei))
  {
    if (ei.bModified)
    {
      if (nSaveType == ST_SIMPLE)
      {
        if (*(wchar_t *)ei.pFile)
        {
          lstrcpynW(sd.szFile, (wchar_t *)ei.pFile, MAX_PATH);
          sd.nCodePage=ei.nCodePage;
          sd.bBOM=ei.bBOM;
          sd.dwFlags=SD_UPDATE;
          return SendMessage(hMainWnd, AKD_SAVEDOCUMENT, (WPARAM)ei.hWndEdit, (LPARAM)&sd);
        }
      }
      else if (nSaveType == ST_NEAR)
      {
        if (*(wchar_t *)ei.pFile)
        {
          wsprintfW(sd.szFile, L"%s.tmp", (wchar_t *)ei.pFile);
          sd.nCodePage=ei.nCodePage;
          sd.bBOM=ei.bBOM;
          sd.dwFlags=0;
          return SendMessage(hMainWnd, AKD_SAVEDOCUMENT, (WPARAM)ei.hWndEdit, (LPARAM)&sd);
        }
      }
      else if (nSaveType == ST_DIR)
      {
        if (*(wchar_t *)ei.pFile)
        {
          wchar_t *wpFileName;

          if (*wszSaveDir)
          {
            CreateDirectoryW(wszSaveDir, NULL);
            wpFileName=GetFileNameW((wchar_t *)ei.pFile);
            wsprintfW(sd.szFile, L"%s\\%s", wszSaveDir, wpFileName);
            sd.nCodePage=ei.nCodePage;
            sd.bBOM=ei.bBOM;
            sd.dwFlags=0;
            return SendMessage(hMainWnd, AKD_SAVEDOCUMENT, (WPARAM)ei.hWndEdit, (LPARAM)&sd);
          }
        }
      }
    }
  }
  return 0;
}

BOOL FileExistsA(char *pFile)
{
  WIN32_FIND_DATAA wfd;
  HANDLE hFind;

  if ((hFind=FindFirstFileA(pFile, &wfd)) == INVALID_HANDLE_VALUE)
    return FALSE;

  FindClose(hFind);
  return TRUE;
}

BOOL FileExistsW(wchar_t *wpFile)
{
  WIN32_FIND_DATAW wfd;
  HANDLE hFind;

  if ((hFind=FindFirstFileW(wpFile, &wfd)) == INVALID_HANDLE_VALUE)
    return FALSE;

  FindClose(hFind);
  return TRUE;
}

char* GetFileNameA(char *pFile)
{
  int i;

  for (i=lstrlenA(pFile) - 1; i >= 0; --i)
    if (pFile[i] == '\\') return (pFile + i + 1);

  return pFile;
}

wchar_t* GetFileNameW(wchar_t *wpFile)
{
  int i;

  for (i=lstrlenW(wpFile) - 1; i >= 0; --i)
    if (wpFile[i] == '\\') return (wpFile + i + 1);

  return wpFile;
}


//// Options

void ReadOptionsA()
{
  PLUGINOPTIONA po;
  HANDLE hOptions;

  if (hOptions=(HANDLE)SendMessage(hMainWnd, AKD_BEGINOPTIONS, POB_READ, (LPARAM)szPluginName))
  {
    po.pOptionName="SaveIntervalEnable";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&bSaveIntervalEnable;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName="SaveInterval";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&dwSaveInterval;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName="SaveFocusEnable";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&bSaveFocusEnable;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName="SaveType";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&nSaveType;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName="SaveDir";
    po.dwType=PO_STRING;
    po.lpData=(LPBYTE)szSaveDir;
    po.dwData=MAX_PATH;
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    SendMessage(hMainWnd, AKD_ENDOPTIONS, (WPARAM)hOptions, 0);
  }
}

void ReadOptionsW()
{
  PLUGINOPTIONW po;
  HANDLE hOptions;

  if (hOptions=(HANDLE)SendMessage(hMainWnd, AKD_BEGINOPTIONS, POB_READ, (LPARAM)wszPluginName))
  {
    po.pOptionName=L"SaveIntervalEnable";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&bSaveIntervalEnable;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName=L"SaveInterval";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&dwSaveInterval;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName=L"SaveFocusEnable";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&bSaveFocusEnable;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName=L"SaveType";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&nSaveType;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName=L"SaveDir";
    po.dwType=PO_STRING;
    po.lpData=(LPBYTE)wszSaveDir;
    po.dwData=MAX_PATH * sizeof(wchar_t);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    SendMessage(hMainWnd, AKD_ENDOPTIONS, (WPARAM)hOptions, 0);
  }
}

void SaveOptionsA()
{
  PLUGINOPTIONA po;
  HANDLE hOptions;

  if (hOptions=(HANDLE)SendMessage(hMainWnd, AKD_BEGINOPTIONS, POB_SAVE, (LPARAM)szPluginName))
  {
    po.pOptionName="SaveIntervalEnable";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&bSaveIntervalEnable;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName="SaveInterval";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&dwSaveInterval;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName="SaveFocusEnable";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&bSaveFocusEnable;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName="SaveType";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&nSaveType;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName="SaveDir";
    po.dwType=PO_STRING;
    po.lpData=(LPBYTE)szSaveDir;
    po.dwData=lstrlenA(szSaveDir) + 1;
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    SendMessage(hMainWnd, AKD_ENDOPTIONS, (WPARAM)hOptions, 0);
  }
}

void SaveOptionsW()
{
  PLUGINOPTIONW po;
  HANDLE hOptions;

  if (hOptions=(HANDLE)SendMessage(hMainWnd, AKD_BEGINOPTIONS, POB_SAVE, (LPARAM)wszPluginName))
  {
    po.pOptionName=L"SaveIntervalEnable";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&bSaveIntervalEnable;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName=L"SaveInterval";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&dwSaveInterval;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName=L"SaveFocusEnable";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&bSaveFocusEnable;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName=L"SaveType";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&nSaveType;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName=L"SaveDir";
    po.dwType=PO_STRING;
    po.lpData=(LPBYTE)wszSaveDir;
    po.dwData=lstrlenW(wszSaveDir) * sizeof(wchar_t) + 2;
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    SendMessage(hMainWnd, AKD_ENDOPTIONS, (WPARAM)hOptions, 0);
  }
}

char* GetLangStringA(LANGID wLangID, int nStringID)
{
  static char szStringBuf[MAX_PATH];

  WideCharToMultiByte(CP_ACP, 0, GetLangStringW(wLangID, nStringID), -1, szStringBuf, MAX_PATH, NULL, NULL);
  return szStringBuf;
}

wchar_t* GetLangStringW(LANGID wLangID, int nStringID)
{
  if (wLangID == LANG_RUSSIAN)
  {
    if (nStringID == STRID_SAVEINTERVAL)
      return L"\x0421\x043E\x0445\x0440\x0430\x043D\x044F\x0442\x044C\x0020\x043A\x0430\x0436\x0434\x044B\x0435";
    if (nStringID == STRID_MIN)
      return L"\x043C\x0438\x043D\x002E";
    if (nStringID == STRID_SAVEFOCUS)
      return L"\x0421\x043E\x0445\x0440\x0430\x043D\x044F\x0442\x044C\x0020\x043F\x0440\x0438\x0020\x043F\x043E\x0442\x0435\x0440\x0435\x0020\x0444\x043E\x043A\x0443\x0441\x0430";
    if (nStringID == STRID_SAVESIMPLE)
      return L"\x0421\x043E\x0445\x0440\x0430\x043D\x044F\x0442\x044C\x0020\x043A\x0430\x043A\x0020\x043E\x0431\x044B\x0447\x043D\x043E";
    if (nStringID == STRID_SAVENEAR)
      return L"\x0421\x043E\x0445\x0440\x0430\x043D\x044F\x0442\x044C\x0020\x0440\x044F\x0434\x043E\x043C\x0020\x043A\x0430\x043A\x0020\x002A\x002E\x0074\x006D\x0070\x0020\x0444\x0430\x0439\x043B";
    if (nStringID == STRID_SAVEDIR)
      return L"\x0421\x043E\x0445\x0440\x0430\x043D\x044F\x0442\x044C\x0020\x0432\x0020\x0434\x0438\x0440\x0435\x043A\x0442\x043E\x0440\x0438\x0438\x003A";
    if (nStringID == STRID_OK)
      return L"\x004F\x004B";
    if (nStringID == STRID_CANCEL)
      return L"\x041E\x0442\x043C\x0435\x043D\x0430";
  }
  else
  {
    if (nStringID == STRID_SAVEINTERVAL)
      return L"Save every";
    if (nStringID == STRID_MIN)
      return L"min.";
    if (nStringID == STRID_SAVEFOCUS)
      return L"Save on lose focus";
    if (nStringID == STRID_SAVESIMPLE)
      return L"Save as usual";
    if (nStringID == STRID_SAVENEAR)
      return L"Save as *.tmp file";
    if (nStringID == STRID_SAVEDIR)
      return L"Save in directory:";
    if (nStringID == STRID_OK)
      return L"OK";
    if (nStringID == STRID_CANCEL)
      return L"Cancel";
  }
  return L"";
}

void InitCommon(PLUGINDATA *pd)
{
  bInitCommon=TRUE;
  hInstanceDLL=pd->hInstanceDLL;
  hMainWnd=pd->hMainWnd;
  hMdiClient=pd->hMdiClient;
  hMainIcon=pd->hMainIcon;
  bOldWindows=pd->bOldWindows;
  bMDI=pd->bMDI;
  wLangSystem=PRIMARYLANGID(pd->wLangSystem);

  if (bOldWindows)
  {
    int i;

    for (i=0; ((char *)pd->pFunction)[i] != ':'; ++i)
      szPluginName[i]=((char *)pd->pFunction)[i];
    szPluginName[i]='\0';
  }
  else
  {
    int i;

    for (i=0; ((wchar_t *)pd->pFunction)[i] != ':'; ++i)
      wszPluginName[i]=((wchar_t *)pd->pFunction)[i];
    wszPluginName[i]='\0';
  }

  if (bOldWindows)
  {
    HMODULE hShell32;

    hShell32=GetModuleHandleA("shell32.dll");
    SHGetPathFromIDListAPrt=(BOOL (WINAPI *)(LPCITEMIDLIST, char *))GetProcAddress(hShell32, "SHGetPathFromIDListA");
    SHBrowseForFolderAPrt=(LPITEMIDLIST (WINAPI *)(LPBROWSEINFOA))GetProcAddress(hShell32, "SHBrowseForFolderA");
    ReadOptionsA();
  }
  else
  {
    HMODULE hShell32;

    hShell32=GetModuleHandleW(L"shell32.dll");
    SHGetPathFromIDListWPrt=(BOOL (WINAPI *)(LPCITEMIDLIST, wchar_t *))GetProcAddress(hShell32, "SHGetPathFromIDListW");
    SHBrowseForFolderWPrt=(LPITEMIDLIST (WINAPI *)(LPBROWSEINFOW))GetProcAddress(hShell32, "SHBrowseForFolderW");
    ReadOptionsW();
  }
}

void InitMain()
{
  bInitMain=TRUE;

  if (bSaveIntervalEnable)
  {
    if (bOldWindows)
      dwSaveTimer=SetTimer(NULL, 0, dwSaveInterval * 60000, (TIMERPROC)TimerProcA);
    else
      dwSaveTimer=SetTimer(NULL, 0, dwSaveInterval * 60000, (TIMERPROC)TimerProcW);
  }

  //SubClass main procedure
  if (bOldWindows)
  {
    NewMainProcData=NULL;
    SendMessage(hMainWnd, AKD_SETMAINPROC, (WPARAM)NewMainProcA, (LPARAM)&NewMainProcData);
  }
  else
  {
    NewMainProcData=NULL;
    SendMessage(hMainWnd, AKD_SETMAINPROC, (WPARAM)NewMainProcW, (LPARAM)&NewMainProcData);
  }
}

void UninitMain()
{
  bInitMain=FALSE;

  //Remove subclass
  if (NewMainProcData)
  {
    SendMessage(hMainWnd, AKD_SETMAINPROC, (WPARAM)NULL, (LPARAM)&NewMainProcData);
    NewMainProcData=NULL;
  }

  if (dwSaveTimer)
  {
    KillTimer(NULL, dwSaveTimer);
    dwSaveTimer=0;
  }
}

//Entry point
extern "C" BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
  if (fdwReason == DLL_PROCESS_ATTACH)
  {
  }
  else if (fdwReason == DLL_THREAD_ATTACH)
  {
  }
  else if (fdwReason == DLL_THREAD_DETACH)
  {
  }
  else if (fdwReason == DLL_PROCESS_DETACH)
  {
    if (bInitMain) UninitMain();
  }
  return TRUE;
}
